package com.jzlife.report.aop;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.jzlife.report.anno.LogRecordFiled;
import com.jzlife.report.dto.SysExportLogDto;
import com.jzlife.report.service.ExportLogRecordService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.util.StreamUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.ServletInputStream;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.lang.reflect.Method;
import java.nio.charset.Charset;
import java.time.LocalDateTime;
import java.util.Map;
import java.util.Objects;

@Aspect
@Slf4j
@Component
public class ExportLogRecordAspect {


    @Resource
    private StringRedisTemplate stringRedisTemplate;


    @Autowired
    private ExportLogRecordService exportLogRecordService;

    @Pointcut("execution(* com.jzlife.report.controller..*.*(..))")
    public void controllerAspect() {

    }

    @Value("${spring.profiles.active}")
    private String Env;

    @Before("controllerAspect()")
    public void before(JoinPoint joinPoint) {
        if (Env.equals("pro")) {
            HttpServletRequest request = ((ServletRequestAttributes) Objects.requireNonNull(RequestContextHolder.getRequestAttributes()))
                    .getRequest();
            MethodSignature methodSignature = (MethodSignature) joinPoint.getSignature();
            Method method = methodSignature.getMethod();
            if (method.isAnnotationPresent(LogRecordFiled.class)) {
                LogRecordFiled annotation = method.getAnnotation(LogRecordFiled.class);
                if (!annotation.isUse()) {
                    log.info(annotation.funName() + "使用了记录日志注解，但是不需要记录日志！");
                    return;
                }
                String token = request.getHeader("token");
                if (StringUtils.isEmpty(token)) {
                    token = request.getParameter("token");
                }
                String IP = getIpAddr(request);
                SysExportLogDto dto = new SysExportLogDto();
                dto.setRequestIp(IP);

                try {
                    Map<String, String[]> parameterMap = request.getParameterMap();
                    ServletInputStream inputStream = request.getInputStream();
                    if (null != inputStream) {
                        String otherParam = StreamUtils.copyToString(inputStream, Charset.defaultCharset());
                        if (parameterMap.containsKey("token")) {
                            JSONObject j1 = JSON.parseObject(otherParam);
                            if (null != j1) {
                                j1.put("token", parameterMap.get("token"));
                                dto.setRequestParm(j1.toJSONString());
                            }
                        } else {
                            dto.setRequestParm(otherParam);
                        }
                    }
                } catch (IOException e) {
                    log.error("读取失败");
                }catch (Exception e){
                    log.error("读取失败");
                }

                dto.setSystemName(annotation.sysName());
                dto.setFunctionName(annotation.funName());
                dto.setRequestUrl(request.getRequestURI());
                String userInfo = stringRedisTemplate.opsForValue().get(token);
                try {
                    JSONObject o = Objects.requireNonNull(JSON.parseArray(userInfo)).getJSONObject(1);
                    dto.setUserId(String.valueOf(o.getInteger("id")));
                    dto.setUserName(o.getString("userName"));
                    log.info("用户id：" + dto.getUserId() +
                            "姓名：" + dto.getUserName() + "在" + LocalDateTime.now() +
                            "时进行了报表查询，" + "查询报表名称:" + dto.getFunctionName() + ",对应的token=" + token);
                    exportLogRecordService.logRecord(dto);
                } catch (Exception e) {
                    log.error("记录报表查询&导出失败，失败原因：" + e.getMessage());
                }
            }
        } else {
            log.info("非生产环境，不记录报表查询日志！");
        }


    }

    public static String getIpAddr(final HttpServletRequest request) {
        if (request == null) {
            return null;
        }
        String ipString = request.getHeader("x-forwarded-for");
        if (StringUtils.isBlank(ipString) || "unknown".equalsIgnoreCase(ipString)) {
            ipString = request.getHeader("Proxy-Client-IP");
        }
        if (StringUtils.isBlank(ipString) || "unknown".equalsIgnoreCase(ipString)) {
            ipString = request.getHeader("WL-Proxy-Client-IP");
        }
        if (StringUtils.isBlank(ipString) || "unknown".equalsIgnoreCase(ipString)) {
            ipString = request.getRemoteAddr();
        }

        // 多个路由时，取第一个非unknown的ip
        final String[] arr = ipString.split(",");
        for (final String str : arr) {
            if (!"unknown".equalsIgnoreCase(str)) {
                ipString = str;
                break;
            }
        }

        return ipString;
    }
}
