package com.jzlife.report.config;

import com.github.xiaoymin.knife4j.spring.annotations.EnableKnife4j;

import java.util.ArrayList;
import java.util.List;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import springfox.documentation.builders.ApiInfoBuilder;
import springfox.documentation.builders.ParameterBuilder;
import springfox.documentation.builders.PathSelectors;
import springfox.documentation.builders.RequestHandlerSelectors;
import springfox.documentation.schema.ModelRef;
import springfox.documentation.service.ApiInfo;
import springfox.documentation.service.Parameter;
import springfox.documentation.spi.DocumentationType;
import springfox.documentation.spring.web.plugins.Docket;
import springfox.documentation.swagger2.annotations.EnableSwagger2;

/**
 * Swagger2API文档的配置
 *
 * @author lu
 * @date 2021/04/01
 */
@Controller
@Configuration
@EnableSwagger2
@EnableKnife4j
public class Swagger2Config {
    @GetMapping("/")
    public String index() {
        return "redirect:swagger-ui.html";
    }

    @Bean
    public Docket createRestApi() {
    	ParameterBuilder token = new ParameterBuilder();
        token.name("token").description("user token")
    	.modelRef(new ModelRef("string")).parameterType("header") 
    	.required(false).build(); //header中的ticket参数非必填，传空也可以
        
        List<Parameter> pars = new ArrayList<Parameter>();  
    	pars.add(token.build()); 

        return new Docket(DocumentationType.SWAGGER_2)
                .apiInfo(apiInfo())
                .select()
                .apis(RequestHandlerSelectors.basePackage("com.jzlife.report.controller"))
                .paths(PathSelectors.any())
                .build()
                .globalOperationParameters(pars);
    }

    private ApiInfo apiInfo() {
        return new ApiInfoBuilder()
                .title("报表系统")
                .description("报表系统")
                .contact("jzreport")
                .version("1.0")
                .build();
    }
}
