package com.jzlife.report.controller;

import com.alibaba.excel.EasyExcel;
import com.jzlife.report.anno.LogRecordFiled;
import com.jzlife.report.dto.ReportProductStocksVo;
import com.jzlife.report.dto.export.BuzlineStockAvailabilityExportDto;
import com.jzlife.report.dto.export.CityBuzServiceorderNumberExport;
import com.jzlife.report.dto.export.CityStationCustomerComplaintExport;
import com.jzlife.report.dto.export.StockStationPersonExportDto;
import com.jzlife.report.entity.reports.HmStationStockAvailability;
import com.jzlife.report.service.report.BjStockStatisticsService;
import com.jzlife.report.util.ExcelListener;
import com.jzlife.report.util.ExcelUtils;
import com.jzlife.report.util.ResponseUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;

@Slf4j
@Api(value = "库存指标相关报表", description = "库存指标相关报表")
@RestController
@RequestMapping("bjStockStatistics")
public class BjStockStatisticsController {

    @Autowired
    BjStockStatisticsService bjStockStatisticsService;

    @ApiOperation(value = "各城市业务线在职情况", httpMethod = "GET", notes = "各城市业务线在职情况yyyy-mm")
    @GetMapping("getStockStationPerson")
    @LogRecordFiled(funName = "各城市业务线在职情况")
    public Object getStockStationPerson(HttpServletRequest request, HttpServletResponse response,
//			@RequestParam(defaultValue = "1") Integer pageNo,
//            @RequestParam(defaultValue = "50") Integer pageSize,
                                        @RequestParam(defaultValue = "false") Boolean isExport, String ofMonth) throws Exception {

        if (StringUtils.isBlank(ofMonth)) {
            return ResponseUtil.fail("数据所属月份不能为空");
        }

        List<StockStationPersonExportDto> list = bjStockStatisticsService.getStockStationPerson(isExport, ofMonth);
        if (isExport) {
            if (list != null && list.size() > 0) {
                ExcelUtils.exportExcel(list, null, "Sheet1", StockStationPersonExportDto.class, "各城市业务线在职情况.xls",
                        response);
            }
        }

        return ResponseUtil.ok(list);
    }

    @ApiOperation(value = "接单情况", httpMethod = "GET", notes = "接单情况yyyy-mm-dd")
    @GetMapping("getSpCityBuzlineServiceorderNumber")
    @LogRecordFiled(funName = "接单情况")
    public Object getSpCityBuzlineServiceorderNumber(HttpServletRequest request, HttpServletResponse response,
                                                     @RequestParam(defaultValue = "false") Boolean isExport, String date) throws Exception {

        if (StringUtils.isBlank(date)) {
            return ResponseUtil.fail("日期不能为空");
        }

        //CityBuzServiceorderNumberExport
        List<CityStationCustomerComplaintExport> list = bjStockStatisticsService.getCityBuzlineServiceorderNumber(date);
        if (isExport) {
            if (list != null && list.size() > 0) {
                ExcelUtils.exportExcel(list, null, "Sheet1", CityStationCustomerComplaintExport.class, "各城市业务线接单情况.xls",
                        response);
            }
        }

        return ResponseUtil.ok(list);
    }

    @ApiOperation(value = "各城市各业务线库存利用率和人均订单", httpMethod = "GET", notes = "各城市各业务线库存利用率和人均订单yyyy-mm")
    @GetMapping("buzlineStockAvailability")
    @LogRecordFiled(funName = "各城市各业务线库存利用率和人均订单")
    public Object sp_report_buzline_stock_availability(HttpServletRequest request, HttpServletResponse response,
                                                       @RequestParam(defaultValue = "false") Boolean isExport, String ofMonth) throws Exception {

        if (StringUtils.isBlank(ofMonth)) {
            return ResponseUtil.fail("数据所属月份不能为空");
        }

        //BuzlineStockAvailabilityExportDto
        List<BuzlineStockAvailabilityExportDto> list = bjStockStatisticsService.buzlineStockAvailability(ofMonth);
        if (isExport) {
            if (list != null && list.size() > 0) {
                ExcelUtils.exportExcel(list, null, "Sheet1", BuzlineStockAvailabilityExportDto.class,
                        "各城市各业务线库存利用率和人均订单.xls", response);
            }
        }

        return ResponseUtil.ok(list);
    }


    @ApiOperation("导入库存利用率-城市-服务站")
    @PostMapping("/importHmStockAvailability")
    @LogRecordFiled(funName = "导入库存利用率-城市-服务站")
    public Object importHmStockAvailability(@RequestParam(value = "file") MultipartFile file,
                                            @RequestParam("ofMonth") String ofMonth) throws IOException {

        if (StringUtils.isBlank(ofMonth)) {
            return ResponseUtil.fail("数据所属月份不能为空");
        }

        ExcelListener<HmStationStockAvailability> listener = new ExcelListener<HmStationStockAvailability>();
        ExcelListener<HmStationStockAvailability> listener2 = new ExcelListener<HmStationStockAvailability>();

        //库存利用率，城市
        EasyExcel.read(file.getInputStream(), HmStationStockAvailability.class, listener).sheet(0).headRowNumber(1).doReadSync();
        List<HmStationStockAvailability> datasCity = listener.getDatas();

        //库存利用率-服务站
        EasyExcel.read(file.getInputStream(), HmStationStockAvailability.class, listener2).sheet(1).headRowNumber(1).doReadSync();
        List<HmStationStockAvailability> datasStation = listener2.getDatas();

        //清空月份数据,再录入数据
        bjStockStatisticsService.importHmStockAvailability(datasCity, datasStation, ofMonth);
        return ResponseUtil.ok();
    }


    @ApiOperation(value = "各城市各服务站不同上线时长保洁师客诉率", httpMethod = "GET", notes = "各城市各服务站不同上线时长保洁师客诉率")
    @GetMapping("getCityBuzServiceorderNumber")
    @LogRecordFiled(funName = "各城市各服务站不同上线时长保洁师客诉率")
    public Object getCityBuzServiceorderNumber(HttpServletRequest request, HttpServletResponse response,
                                               @RequestParam(defaultValue = "false") Boolean isExport, String startDate, String endDate) throws Exception {


        if (StringUtils.isBlank(startDate) || StringUtils.isBlank(endDate)) {
            return ResponseUtil.fail("日期范围不能为空");
        }

        List<CityBuzServiceorderNumberExport> list = bjStockStatisticsService.getCityBuzServiceorderNumber(startDate,
                endDate);
        if (isExport) {
            if (list != null && list.size() > 0) {
                ExcelUtils.exportExcel(list, null, "Sheet1", CityBuzServiceorderNumberExport.class,
                        "各城市各服务站不同上线时长保洁师客诉率.xls", response);
            }
        }

        return ResponseUtil.ok(list);
    }

    @ApiOperation(value = "各城市商品库存剩余率", httpMethod = "GET", notes = "各城市商品库存剩余率")
    @GetMapping("getCityProductStock")
    @LogRecordFiled(funName = "各城市商品库存剩余率")
    public Object getCityProductStock(HttpServletRequest request, HttpServletResponse response,
                                      @RequestParam(defaultValue = "false") Boolean isExport, String city) throws Exception {


        if (StringUtils.isBlank(city)) {
            return ResponseUtil.fail("城市不能为空");
        }

        List<ReportProductStocksVo> list = bjStockStatisticsService.getCityProductStock(city);
        if (isExport) {
            if (list != null && list.size() > 0) {
                ExcelUtils.exportExcel(list, null, "Sheet1", ReportProductStocksVo.class,
                        "各城市商品库存剩余率.xls", response);
                return null;
            }
        }

        return ResponseUtil.ok(list);
    }

}
