package com.jzlife.report.controller;

import cn.afterturn.easypoi.excel.entity.params.ExcelExportEntity;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jzlife.report.anno.LogRecordFiled;
import com.jzlife.report.dao.manager.CallTaskMapper;
import com.jzlife.report.dao.reports.CrmCampaignMapper;
import com.jzlife.report.dto.ConsultTypeDetailReport;
import com.jzlife.report.dto.OBCallTask;
import com.jzlife.report.dto.OBCallTaskDetailVo;
import com.jzlife.report.dto.douyin.DouyinFianceVo;
import com.jzlife.report.util.ExcelUtils;
import com.jzlife.report.util.ResponseUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.NotNull;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Slf4j
@Api(value = "电访任务", description = "ob电访任务")
@RestController
@RequestMapping("callTask")
public class CallTaskController {

    @Autowired
    private CrmCampaignMapper crmCampaignMapper;

    @Autowired
    private CallTaskMapper callTaskMapper;


    @ApiOperation(value = "获取所有外拨任务名称", httpMethod = "GET", notes = "查询所有坐席")
    @GetMapping("getAllSysUser")
    @LogRecordFiled(funName = "查询所有坐席")
    public Object getAllSysUser(String category) {
        return ResponseUtil.ok(callTaskMapper.selectAllUser());
    }

    @ApiOperation(value = "获取所有外拨任务名称", httpMethod = "GET", notes = "查询所有任务")
    @GetMapping("getAllCampaign")
    @LogRecordFiled(funName = "获取所有任务名称")
    public Object getAllCampaign(String category) {
        return ResponseUtil.ok(crmCampaignMapper.selectAll(category));
    }

    @ApiOperation(value = "电访任务报表", httpMethod = "GET", notes = "电访任务报表")
    @GetMapping("getCallTask")
    @LogRecordFiled(funName = "电访任务报表")
    public Object getCallTask(String startTime, String endTime, Integer campaignId) {
        return ResponseUtil.ok(callTaskMapper.selectCallTask(startTime, endTime, campaignId));
    }

    @ApiOperation(value = "IB电访结果报表", httpMethod = "GET", notes = "IB电访结果报表")
    @GetMapping("getCallIB")
    @LogRecordFiled(funName = "IB电访结果报表")
    public Object getCallIB(String startTime, String endTime) {
        return ResponseUtil.ok(callTaskMapper.selectIB(startTime, endTime));
    }

    @ApiOperation(value = "电访工单咨询/投诉项明细报表", httpMethod = "GET", notes = "电访任务工单明细报表")
    @LogRecordFiled(funName = "电访工单明细报表")
    @GetMapping("/ticketConsultDetailList")
    public Object ticketConsultDetailList(@RequestParam(required = false) Integer campaignId,
                                          @NotNull @RequestParam(required = false) String startTime,
                                          @NotNull @RequestParam(required = false) String endTime) {

        Long days = LocalDate.parse(endTime).toEpochDay() - LocalDate.parse(startTime).toEpochDay();
        if (days > 61)
            return ResponseUtil.fail(3, "查询时间不能大于二个月");

        List<ConsultTypeDetailReport> consultTypeData = callTaskMapper.ticketConsultTypeDetailReport(campaignId, startTime, endTime);

        return ResponseUtil.ok(consultTypeData);
    }

    @ApiOperation(value = "ob电访拨打情况报表", httpMethod = "GET", notes = "ob电访拨打情况报表")
    @GetMapping("callResult")
    @LogRecordFiled(funName = "ob电访拨打情况报表")
    public Object callResult(String startTime, String endTime, String campaigns) {
        return ResponseUtil.ok(callTaskMapper.selectCallResult(startTime, endTime, campaigns));
    }

    @ApiOperation(value = "ob电访任务看板", httpMethod = "GET", notes = "ob电访任务看板")
    @GetMapping("callTaskBoard")
    @LogRecordFiled(funName = "ob电访任务看板")
    public Object callTaskBoard(String startTime, String endTime, String city) {
        List<OBCallTask> list = callTaskMapper.selectCallTaskBoard(startTime, endTime, city);
        if (list != null && list.size() > 0) {
            for (OBCallTask callTask : list) {
                callTask.setDetailList(callTaskMapper.selectCallTaskDetailBoard(startTime, endTime, callTask.getCampaignId()));
            }
        }
        return ResponseUtil.ok(list);
    }

    @ApiOperation(value = "企微回访", httpMethod = "GET", notes = "企微回访")
    @GetMapping("qywxCall")
    @LogRecordFiled(funName = "企微回访")
    public Object qywxCall(@RequestParam("startTime") String startTime, @RequestParam("endTime") String endTime, String city, Integer pageNum, Integer pageSize, Boolean export, HttpServletResponse response) {
        if (export) {
            //开始结束时间不能超过一个月
            Long days = LocalDate.parse(endTime).toEpochDay() - LocalDate.parse(startTime).toEpochDay();
            if (days > 31) {
                return ResponseUtil.fail(3, "查询时间不能大于一个月");
            }
            Page<Map<String, Object>> maps = callTaskMapper.selectQywxCall(startTime, endTime, city);
            if (!maps.isEmpty()) {
                List<ExcelExportEntity> beanList = new ArrayList<>();
                beanList.add(new ExcelExportEntity("城市", "城市"));
                beanList.add(new ExcelExportEntity("任务名称", "任务名称"));
                beanList.add(new ExcelExportEntity("作业单号", "作业单号"));
                beanList.add(new ExcelExportEntity("服务时间", "服务时间"));
                beanList.add(new ExcelExportEntity("回访方式", "回访方式"));
                beanList.add(new ExcelExportEntity("回访时间", "回访时间"));
                beanList.add(new ExcelExportEntity("回访结果", "回访结果"));
                beanList.add(new ExcelExportEntity("备注", "备注"));
                ExcelUtils.writeExcelToStream("企微回访.xlsx", beanList, maps, response);
            }
            return null;
        }
        if (null == pageNum) {
            pageNum = 1;
        }
        if (null == pageSize) {
            pageSize = 30;
        }
        PageHelper.startPage(pageNum, pageSize);
        Page<Map<String, Object>> page = callTaskMapper.selectQywxCall(startTime, endTime, city);
        return ResponseUtil.ok(new PageInfo<>(page));
    }

    @ApiOperation(value = "电访结果报表", httpMethod = "GET", notes = "外呼电访明细报表")
    @GetMapping("callTaskDetail")
    @LogRecordFiled(funName = "外呼电访明细报表")
    public Object getCallTaskDetail(@RequestParam("startTime") String startTime,
                                    @RequestParam("endTime") String endTime,
                                    @RequestParam(required = true) List<Integer> users,
                                    Boolean isExport,
                                    Integer pageNum, Integer pageSize, HttpServletResponse response) throws Exception {
        if (isExport == null || isExport == false) {
            PageHelper.startPage(pageNum, pageSize);
            List<OBCallTaskDetailVo> list = callTaskMapper.selectOBCallDetailList(startTime, endTime,users);
            return ResponseUtil.ok(new PageInfo<>(list));
        } else {
            List<OBCallTaskDetailVo> list = callTaskMapper.selectOBCallDetailList(startTime, endTime,users);
            ExcelUtils.exportExcel(list, null, "Sheet1", OBCallTaskDetailVo.class, "callTaskDetail.xls", response);
            return null;
        }
    }

}
