package com.jzlife.report.controller;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jzlife.report.anno.LogRecordFiled;
import com.jzlife.report.dao.kaocf.KaocfMapper;
import com.jzlife.report.dto.kaocf.KaoCom;
import com.jzlife.report.dto.kaocf.KaoSales;
import com.jzlife.report.util.ExcelUtils;
import com.jzlife.report.util.ResponseUtil;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 花王报表相关
 */

@RestController
@RequestMapping("kaocf")
public class KAOCFController {

    @Resource
    private KaocfMapper kaocfMapper;

    @ApiOperation(value = "花王报表", httpMethod = "GET", notes = "销售报表")
    @GetMapping("getKaoSales")
    @LogRecordFiled(funName = "销售报表")
    public Object getKaoSales(
            @RequestParam("startTime") String startDate,
            @RequestParam("endTime") String endDate,
            Boolean isExport,
            Integer page, Integer limit, HttpServletResponse response) throws Exception {
        if (isExport == null || isExport == false) {
            PageHelper.startPage(page, limit);
            List<KaoSales> list = kaocfMapper.getKaoSales(startDate, endDate, null);
            return ResponseUtil.ok(new PageInfo<>(list));
        } else {
            List<KaoSales> list = kaocfMapper.getKaoSales(startDate, endDate, null);
            ExcelUtils.exportExcel(list, null, "Sheet1", KaoSales.class, "douyinpage.xls", response);
            return null;
        }
    }

    @ApiOperation(value = "花王报表", httpMethod = "GET", notes = "履约报表")
    @GetMapping("getKaoCom")
    @LogRecordFiled(funName = "履约报表")
    public Object getKaoCom(
            @RequestParam("startTime") String startDate,
            @RequestParam("endTime") String endDate,
            Boolean isExport,
            Integer page, Integer limit, HttpServletResponse response) throws Exception {
        if (isExport == null || isExport == false) {
            PageHelper.startPage(page, limit);
            List<KaoCom> list = kaocfMapper.getKaoCom(startDate, endDate);
            return ResponseUtil.ok(new PageInfo<>(list));
        } else {
            List<KaoCom> list = kaocfMapper.getKaoCom(startDate, endDate);
            ExcelUtils.exportExcel(list, null, "Sheet1", KaoCom.class, "douyinpage.xls", response);
            return null;
        }
    }
}
