package com.jzlife.report.controller;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jzlife.report.dto.*;
import com.jzlife.report.dto.douyin.BookVerifyDetailVo;
import com.jzlife.report.dto.douyin.DouyinBookTotalByFraVo;
import com.jzlife.report.dto.douyin.DouyinVerifyDetailByTanletVo;
import com.jzlife.report.dto.douyin.DouyinVerifyDetailFraVo;
import com.jzlife.report.service.report.OrderReservationVerificationService;
import com.jzlife.report.util.ExcelUtils;
import com.jzlife.report.util.ResponseUtil;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

@RestController
@RequestMapping(value = "reservation")
public class OrderReservationVerificationController {
    @Autowired
    OrderReservationVerificationService orderReservationVerificationService;

    @ApiOperation("当日预约率表")
    @PostMapping("getOrderReservationRate")
    public Object getOrderReservationRate(@RequestBody OrderReservationQueryDto queryDto, HttpServletResponse response) throws Exception {
        if (queryDto.getIsExport() == null || !queryDto.getIsExport()) {
            PageHelper.startPage(queryDto.getPage(), queryDto.getLimit());
            List<OrderReservationRateDto> list = orderReservationVerificationService.getOrderReservationRate(queryDto);
            return ResponseUtil.ok(new PageInfo<>(list));
        } else {
            List<OrderReservationRateDto> list = orderReservationVerificationService.getOrderReservationRate(queryDto);
            ExcelUtils.exportExcel(list, null, "sheet1", OrderReservationRateDto.class, "当日预约率.xls", response);
            return null;
        }
    }

    @ApiOperation("预约后退款--订单维度")
    @PostMapping("getRefundAfterReservation")
    public Object getRefundAfterReservation(@RequestBody OrderReservationQueryDto queryDto, HttpServletResponse response) throws Exception {
        if (queryDto.getIsExport() == null || !queryDto.getIsExport()) {
            PageHelper.startPage(queryDto.getPage(), queryDto.getLimit());
            List<RefundAfterReservationDto> list = orderReservationVerificationService.getRefundAfterReservation(queryDto);
            return ResponseUtil.ok(new PageInfo<>(list));
        } else {
            List<RefundAfterReservationDto> list = orderReservationVerificationService.getRefundAfterReservation(queryDto);
            ExcelUtils.exportExcel(list, null, "sheet1", RefundAfterReservationDto.class, "预约后退款.xls", response);
            return null;
        }
    }

    @ApiOperation("预约后退款--预约维度")
    @PostMapping("getRefundAfterByBookDate")
    public Object getRefundAfterByBookDate(@RequestBody OrderReservationQueryDto queryDto, HttpServletResponse response) throws Exception {
        if (queryDto.getIsExport() == null || !queryDto.getIsExport()) {
            PageHelper.startPage(queryDto.getPage(), queryDto.getLimit());
            List<RefundAfterBookDateDto> list = orderReservationVerificationService.getRefundAfterByBookDate(queryDto);
            return ResponseUtil.ok(new PageInfo<>(list));
        } else {
            List<RefundAfterBookDateDto> list = orderReservationVerificationService.getRefundAfterByBookDate(queryDto);
            ExcelUtils.exportExcel(list, null, "sheet1", RefundAfterBookDateDto.class, "预约后退款.xls", response);
            return null;
        }
    }

    @ApiOperation("核销率周报表")
    @PostMapping("getVerificationRateWeek")
    public Object getVerificationRateWeek(@RequestBody OrderReservationQueryDto queryDto, HttpServletResponse response) throws Exception {
        if (queryDto.getIsExport() == null || !queryDto.getIsExport()) {
            PageHelper.startPage(queryDto.getPage(), queryDto.getLimit());
            List<VerificationRateWeekDto> list = orderReservationVerificationService.getVerificationRateWeek(queryDto);
            return ResponseUtil.ok(new PageInfo<>(list));
        } else {
            List<VerificationRateWeekDto> list = orderReservationVerificationService.getVerificationRateWeek(queryDto);
            ExcelUtils.exportExcel(list, null, "sheet1", VerificationRateWeekDto.class, "核销率周报表.xls", response);
            return null;
        }
    }

    @ApiOperation("核销率月报表")
    @PostMapping("getVerificationRateMonth")
    public Object getVerificationRateMonth(@RequestBody OrderReservationQueryDto queryDto, HttpServletResponse response) throws Exception {
        if (queryDto.getIsExport() == null || !queryDto.getIsExport()) {
            PageHelper.startPage(queryDto.getPage(), queryDto.getLimit());
            List<VerificationRateMonthDto> list = orderReservationVerificationService.getVerificationRateMonth(queryDto);
            return ResponseUtil.ok(new PageInfo<>(list));
        } else {
            List<VerificationRateMonthDto> list = orderReservationVerificationService.getVerificationRateMonth(queryDto);
            ExcelUtils.exportExcel(list, null, "sheet1", VerificationRateMonthDto.class, "核销率月报表.xls", response);
            return null;
        }
    }

    @ApiOperation("待联系工单跟进表")
    @PostMapping("getContactOrder")
    public Object getContactOrder(@RequestBody OrderReservationQueryDto queryDto, HttpServletResponse response) throws Exception {
        if (queryDto.getIsExport() == null || !queryDto.getIsExport()) {
            PageHelper.startPage(queryDto.getPage(), queryDto.getLimit());
            List<ContactOrderDto> list = orderReservationVerificationService.getContactOrder(queryDto);
            return ResponseUtil.ok(new PageInfo<>(list));
        } else {
            List<ContactOrderDto> list = orderReservationVerificationService.getContactOrder(queryDto);
            ExcelUtils.exportExcel(list, null, "sheet1", ContactOrderDto.class, "待联系工单跟进表.xls", response);
            return null;
        }
    }

    @ApiOperation("核销明细报表--加盟商维度")
    @PostMapping("getVerifyDetailByBookDate")
    public Object getVerifyDetailByBookDate(@RequestBody OrderReservationQueryDto queryDto, HttpServletResponse response) throws Exception {
        if (queryDto.getIsExport() == null || !queryDto.getIsExport()) {
            PageHelper.startPage(queryDto.getPage(), queryDto.getLimit());
            List<DouyinVerifyDetailFraVo> list = orderReservationVerificationService.getVerifyDetailByBookDate(queryDto);
            return ResponseUtil.ok(new PageInfo<>(list));
        } else {
            List<DouyinVerifyDetailFraVo> list = orderReservationVerificationService.getVerifyDetailByBookDate(queryDto);
            ExcelUtils.exportExcel(list, null, "sheet1", DouyinVerifyDetailFraVo.class, "核销明细.xls", response);
            return null;
        }
    }

    @ApiOperation("核销明细报表--达人维度")
    @PostMapping("getVerifyDetailByTanlet")
    public Object getVerifyDetailByTanlet(@RequestBody OrderReservationQueryDto queryDto, HttpServletResponse response) throws Exception {
        if (queryDto.getIsExport() == null || !queryDto.getIsExport()) {
            PageHelper.startPage(queryDto.getPage(), queryDto.getLimit());
            List<DouyinVerifyDetailByTanletVo> list = orderReservationVerificationService.getVerifyDetailByTanlet(queryDto);
            return ResponseUtil.ok(new PageInfo<>(list));
        } else {
            List<DouyinVerifyDetailByTanletVo> list = orderReservationVerificationService.getVerifyDetailByTanlet(queryDto);
            ExcelUtils.exportExcel(list, null, "sheet1", DouyinVerifyDetailByTanletVo.class, "核销明细.xls", response);
            return null;
        }
    }

    @ApiOperation("核销预约明细报表")
    @PostMapping("getBookVerifyDetailList")
    public Object getBookVerifyDetailList(@RequestBody OrderReservationQueryDto queryDto, HttpServletResponse response) throws Exception {
        if (queryDto.getIsExport() == null || !queryDto.getIsExport()) {
            PageHelper.startPage(queryDto.getPage(), queryDto.getLimit());
            List<BookVerifyDetailVo> list = orderReservationVerificationService.getVerifyDetailList(queryDto);
            return ResponseUtil.ok(new PageInfo<>(list));
        } else {
            List<BookVerifyDetailVo> list = orderReservationVerificationService.getVerifyDetailList(queryDto);
            ExcelUtils.exportExcel(list, null, "sheet1", BookVerifyDetailVo.class, "核销明细.xls", response);
            return null;
        }
    }


    @ApiOperation("预约汇总报表by fra")
    @PostMapping("getBookTotalListByFra")
    public Object getBookTotalListByFra(@RequestBody OrderReservationQueryDto queryDto, HttpServletResponse response) throws Exception {
        if (queryDto.getIsExport() == null || !queryDto.getIsExport()) {
            PageHelper.startPage(queryDto.getPage(), queryDto.getLimit());
            List<DouyinBookTotalByFraVo> list = orderReservationVerificationService.getBookTotalByFra(queryDto);
            return ResponseUtil.ok(new PageInfo<>(list));
        } else {
            List<DouyinBookTotalByFraVo> list = orderReservationVerificationService.getBookTotalByFra(queryDto);
            ExcelUtils.exportExcel(list, null, "sheet1", DouyinBookTotalByFraVo.class, "核销明细.xls", response);
            return null;
        }
    }

}
