package com.jzlife.report.controller;


import cn.afterturn.easypoi.excel.entity.params.ExcelExportEntity;
import com.jzlife.report.anno.LogRecordFiled;
import com.jzlife.report.service.ReBuy2Service;
import com.jzlife.report.util.ExcelUtils;
import com.jzlife.report.util.ResponseUtil;
import com.jzlife.report.vo.ReBuyDetailVo;
import com.jzlife.report.vo.ReBuyQueryVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("reBuy")
@Api(value = "ReBuyController", description = "复购看板")
public class ReBuy2Controller {

    @Autowired
    private ReBuy2Service reBuy2Service;


    @ApiOperation(value = "复购-折线图")
    @PostMapping("reBuyLine")
    @LogRecordFiled(funName = "复购看板-折线图")
    public Object reBuyLine(@Valid @RequestBody ReBuyQueryVo vo, HttpServletRequest request) {
        vo.setUserId(Integer.parseInt(request.getHeader("userId")));
        return ResponseUtil.ok(reBuy2Service.reBuyLine(vo));
    }

    @ApiOperation(value = "复购-详情")
    @PostMapping("reBuyDetail")
    @LogRecordFiled(funName = "复购看板-详情")
    public Object reBuyDetail(@Valid @RequestBody ReBuyDetailVo vo, HttpServletRequest request) {
        vo.setUserId(Integer.parseInt(request.getHeader("userId")));
        return ResponseUtil.ok(reBuy2Service.reBuyDetail(vo));
    }


    @ApiOperation(value = "复购-工单号跟进历史")
    @GetMapping("bookTradeOrderHistory")
    @LogRecordFiled(funName = "复购看板-工单号跟进历史")
    public Object bookTradeOrderHistory(@RequestParam("bookTradeOrder") String bookTradeOrder) {
        return ResponseUtil.ok(reBuy2Service.bookTradeOrderHistory(bookTradeOrder));
    }


    @ApiOperation(value = "复购-详情导出")
    @PostMapping("reBuyDetailExport")
    @LogRecordFiled(funName = "复购看板-详情导出")
    public void reBuyDetailExport(@Valid @RequestBody ReBuyDetailVo vo, HttpServletRequest request, HttpServletResponse response) {
        vo.setUserId(Integer.parseInt(request.getHeader("userId")));
        vo.setIsExport(true);
        List<Map<String, Object>> o = (List<Map<String, Object>>) reBuy2Service.reBuyDetail(vo);
        if (o.size() > 0) {
            List<ExcelExportEntity> beanList = new ArrayList<>();
            if (vo.getShowType() == 1) {
                //工单维度
                beanList.add(new ExcelExportEntity("城市", "city"));
                beanList.add(new ExcelExportEntity("驿站", "jz_station"));
                beanList.add(new ExcelExportEntity("服务站", "hm_station"));
                beanList.add(new ExcelExportEntity("商品分类", "trade_type"));
                beanList.add(new ExcelExportEntity("服务项", "trade_goods_type"));
                beanList.add(new ExcelExportEntity("履约时间", "begin_date"));
                beanList.add(new ExcelExportEntity("工单号", "book_trade_no"));
                beanList.add(new ExcelExportEntity("商家运营", "worker_manager"));
                beanList.add(new ExcelExportEntity("商家编号", "worker_no"));
                beanList.add(new ExcelExportEntity("商家姓名", "worker_name"));
                beanList.add(new ExcelExportEntity("是否复购", "is_rebuy"));
                beanList.add(new ExcelExportEntity("复购分类", "rebuy_category"));
                beanList.add(new ExcelExportEntity("实际复购人", "rebuy_worker"));
                beanList.add(new ExcelExportEntity("复购商品项", "rebuy_goods"));

            } else {
                //服务者维度
                beanList.add(new ExcelExportEntity("城市", "city"));
                beanList.add(new ExcelExportEntity("驿站", "post_name"));
                beanList.add(new ExcelExportEntity("服务站", "station"));
                beanList.add(new ExcelExportEntity("商品分类", "trade_type"));
                beanList.add(new ExcelExportEntity("商家运营", "worker_manager"));
                beanList.add(new ExcelExportEntity("商家编号", "worker_no"));
                beanList.add(new ExcelExportEntity("商家姓名", "worker_name"));
                beanList.add(new ExcelExportEntity("商家手机号", "worker_tel"));
                beanList.add(new ExcelExportEntity("上线时间", "entry_time"));
                beanList.add(new ExcelExportEntity("复购数", "rebuy_count"));
                beanList.add(new ExcelExportEntity("总服务数", "total_count"));
                beanList.add(new ExcelExportEntity("复购率", "rebuy_rate"));
                beanList.add(new ExcelExportEntity("警戒值", "config_value"));
                beanList.add(new ExcelExportEntity("标签", "tag"));
            }
            ExcelUtils.writeExcelToStream("复购详情.xlsx", beanList, o, response);
        }
    }
}
