package com.jzlife.report.controller;

import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.jzlife.report.anno.LogRecordFiled;
import com.jzlife.report.dao.douyin.DouyinBoardMapper;
import com.jzlife.report.dao.reports.ReportServerGmvBoardMapper;
import com.jzlife.report.dto.EchartsLine;
import com.jzlife.report.dto.douyin.DouyinBoardBookVo;
import com.jzlife.report.dto.douyin.DouyinBoardLineVo;
import com.jzlife.report.dto.douyin.DouyinBoardMapVo;
import com.jzlife.report.dto.douyin.DouyinBoardSalesVo;
import com.jzlife.report.service.redis.RedisService;
import com.jzlife.report.util.DateUtil;
import com.jzlife.report.util.DateUtils;
import com.jzlife.report.util.ResponseUtil;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.Duration;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 抖音看板相关报表
 */
@RestController
@RequestMapping("serviceGmv")
public class ServiceGmvController {

    @Resource
    private ReportServerGmvBoardMapper boardMapper;

    @Autowired
    private RedisService redisService;

    @ApiOperation(value = "服务Gmv看板相关报表", httpMethod = "GET", notes = "获取今日&本月服务gmv数据")
    @GetMapping("gmv")
    @LogRecordFiled(funName = "获取今日&本月服务gmv数据")
    public Object getToday(String channel) throws Exception {
        Map<String, Object> obj = new HashMap<String, Object>();
        obj.put("refreshTime", LocalDateTime.now());

        String redisKey_Day = "ServiceGmvBoardSalesVo_Day_" + channel;

        //日销售
        DouyinBoardSalesVo boardBookVo_Day;
        String value = redisService.get(redisKey_Day);
        if (StringUtils.isNotEmpty(value))
            boardBookVo_Day = JSONObject.parseObject(value, DouyinBoardSalesVo.class);
        else {
            boardBookVo_Day = boardMapper.getTodaySales(channel);
            Double gmvY = boardMapper.getYesterdaySales(channel);//获取昨日gmv
            if (gmvY != null)
                boardBookVo_Day.setGmvY(BigDecimal.valueOf(gmvY));
            //客单价设置
            BigDecimal bjOrders = new BigDecimal(boardBookVo_Day.getBjOrders() == null ? "0" : boardBookVo_Day.getBjOrders().toString());
            BigDecimal plOrders = new BigDecimal(boardBookVo_Day.getPlOrders() == null ? "0" : boardBookVo_Day.getPlOrders().toString());

            if (!bjOrders.equals(BigDecimal.ZERO)) {
                boardBookVo_Day.setBjPrice(boardBookVo_Day.getBjGmv().divide(bjOrders, 2, RoundingMode.HALF_UP));
            }
            if (!plOrders.equals(BigDecimal.ZERO)) {
                boardBookVo_Day.setPlPrice(boardBookVo_Day.getPlGmv().divide(plOrders, 2, RoundingMode.HALF_UP));
            }

            Duration duration = Duration.between(LocalDateTime.now(), LocalDateTime.now().plusMinutes(1));
            redisService.set(redisKey_Day, JSONObject.toJSONString(boardBookVo_Day), duration);
        }

        //月销售
        String redisKey_Month = "ServiceGmvBoardSalesVo_Month" + channel;
        DouyinBoardSalesVo boardSalesVo_Month;
        String monthStr = redisService.get(redisKey_Month);
        if (StringUtils.isNotEmpty(monthStr))
            boardSalesVo_Month = JSONObject.parseObject(monthStr, DouyinBoardSalesVo.class);
        else {
            boardSalesVo_Month = getMonth(channel);

            Duration duration = Duration.between(LocalDateTime.now(), LocalDateTime.now().plusMinutes(10));
            redisService.set(redisKey_Month, JSONObject.toJSONString(boardSalesVo_Month), duration);
        }

        obj.put("day", boardBookVo_Day);
        obj.put("month", boardSalesVo_Month);
        return ResponseUtil.ok(obj);
    }

    /**
     * 获取月销售
     *
     * @param channel
     * @return
     * @throws Exception
     */
    public DouyinBoardSalesVo getMonth(String channel) throws Exception {


        DouyinBoardSalesVo monthBoardBookVo = boardMapper.getMonthSales(channel);

        //获取上月销售GMV
        String redisKey_LastMonth = "ServiceGmvBoardSalesVo_LastMonth_" + channel;
        String lastMonthStr = redisService.get(redisKey_LastMonth);

        Double monthGmv;
        if (StringUtils.isNotEmpty(lastMonthStr))
            monthGmv = JSONObject.parseObject(lastMonthStr, Double.class);
        else {
            monthGmv = boardMapper.getLastMonthSales(channel);//获取上月gmv累计
            redisService.set(redisKey_LastMonth, monthGmv == null ? "0" : monthGmv.toString(), Duration.between(LocalDateTime.now(), LocalDateTime.now().plusHours(12)));
        }

        //赋值
        monthBoardBookVo.setGmvY(BigDecimal.valueOf(monthGmv));

        //客单价设置
        BigDecimal bjOrders = new BigDecimal(monthBoardBookVo.getBjOrders() == null ? "0" : monthBoardBookVo.getBjOrders().toString());
        BigDecimal plOrders = new BigDecimal(monthBoardBookVo.getPlOrders() == null ? "0" : monthBoardBookVo.getPlOrders().toString());
        if (bjOrders.compareTo(BigDecimal.ZERO) > 0)
            monthBoardBookVo.setBjPrice(monthBoardBookVo.getBjGmv().divide(bjOrders, 2, RoundingMode.HALF_UP));

        if (plOrders.compareTo(BigDecimal.ZERO) > 0)
            monthBoardBookVo.setPlPrice(monthBoardBookVo.getPlGmv().divide(plOrders, 2, RoundingMode.HALF_UP));

        return monthBoardBookVo;
    }

    @ApiOperation(value = "服务Gmv看板相关报表", httpMethod = "GET", notes = "近30天每日服务情况")
    @GetMapping("lineBookData")
    @LogRecordFiled(funName = "近30天每日服务情况")
    public Object lineBookData(String channel, String type) throws Exception {
        //type 日维度 月维度
        //定义4条曲线
        EchartsLine echOrdersList = new EchartsLine("line");//订单量
        echOrdersList.setName("单量");
        echOrdersList.setData(Lists.newArrayList());

        EchartsLine gmv = new EchartsLine("bar");//gmv
        gmv.setName("GMV");
        gmv.setStack("Total");
        gmv.setData(Lists.newArrayList());

        String redisKey = "ServiceGmvBoardBookVo_" + channel + "_" + type;

        List<DouyinBoardBookVo> list = Lists.newArrayList();
        String value = redisService.get(redisKey);
        if (StringUtils.isNotEmpty(value))
            list = JSONObject.parseArray(value, DouyinBoardBookVo.class);
        else {
            list = boardMapper.getServiceBookDataForLine(channel, type);
            Duration duration = Duration.between(LocalDateTime.now(), LocalDateTime.now().plusMinutes(60));
            redisService.set(redisKey, JSONObject.toJSONString(list), duration);
        }

        List<String> dayList = Lists.newArrayList();
        //近30天日期格式化
        if (type.equals("日维度")) {
            for (int i = 30; i > 0; i--) {
                String date = DateUtils.format.format(DateUtil.addDay(new Date(), -i));
                dayList.add(date);

                List<DouyinBoardBookVo> dList = list.stream().filter(item -> item.getDate().equals(date)).collect(Collectors.toList());
                echOrdersList.getData().add(dList.size() > 0 ? dList.get(0).getDirectCount() : 0);
                gmv.getData().add(dList.size() > 0 ? dList.get(0).getGmv() : 0);
            }
        } else {
            for (int i = 12; i >= 0; i--) {
                String date = DateUtils.format.format(DateUtil.addMonth(new Date(), -i)).substring(0, 7);
                dayList.add(date);

                List<DouyinBoardBookVo> dList = list.stream().filter(item -> item.getDate().equals(date)).collect(Collectors.toList());
                echOrdersList.getData().add(dList.size() > 0 ? dList.get(0).getDirectCount() : 0);
                gmv.getData().add(dList.size() > 0 ? dList.get(0).getGmv() : 0);
            }

        }

        List<EchartsLine> echLiss = Lists.newArrayList();
        echLiss.add(echOrdersList);
        echLiss.add(gmv);

        Map<String, Object> data = new HashMap<String, Object>();
        data.put("series", echLiss);
        data.put("days", dayList);

        return ResponseUtil.ok(data);
    }

}
