package com.jzlife.report.filter;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.jzlife.report.config.mvc.BodyReaderRequestFilter;
import com.jzlife.report.config.mvc.RequestWrapper;
import com.jzlife.report.exception.PromiseException;
import com.jzlife.report.service.newreport.PrivilegeManService;
import org.apache.commons.lang3.StringUtils;
import org.apache.tomcat.util.http.MimeHeaders;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.lang.reflect.Field;
import java.util.Map;


/**
 * 城市服务站权限设置
 */
@Component
public class CityStationFilter extends BodyReaderRequestFilter {

    @Autowired
    private PrivilegeManService privilegeManService;

    @Override
    public void doFilter(ServletRequest req, ServletResponse res, FilterChain filterChain) throws IOException, ServletException {
        HttpServletRequest request = (HttpServletRequest) req;
        String url = request.getRequestURI();
        RequestWrapper requestWrapper = new RequestWrapper(request);
        if (url.contains("/login") | url.contains("error") | url.contains("syncMail")||url.contains("swagger")||url.contains("api-docs")) {
            filterChain.doFilter(requestWrapper, res);
            return;
        }
        Integer userId = (Integer) requestWrapper.getAttribute("userId");
        reflectSetHeader(request, "userId", userId.toString());
        boolean all = privilegeManService.getPrivilegeManByUserId(userId);
        //是所有权限，不对城市做校验
        Map<String, String> citys = privilegeManService.getPrivilegeByUserIdAndType(userId, "a");
        if (all) {
//            if (null != citys && citys.containsKey("data_arr")) {
//                String allowCity = citys.get("data_name").replace("市", "").replace("全国", "");
//                if (allowCity.startsWith(",")) {
//                    allowCity = allowCity.substring(1);
//                }
//                reflectSetHeader(request, "allowCity", allowCity);
//            }
        } else {
            // 查询拥有的城市、驿站、服务站权限
            if (null != citys && citys.containsKey("data_arr")) {
                String allowCity = citys.get("data_name").replace("市", "");
                reflectSetHeader(request, "allowCity", allowCity);
            }
            Map<String, String> posts = privilegeManService.getPrivilegeByUserIdAndType(userId, "b");
            Map<String, String> stations = privilegeManService.getPrivilegeByUserIdAndType(userId, "c");
            if (((HttpServletRequest) req).getMethod().equals("POST")) {
                String body = requestWrapper.getBody();
                if (body != null && !body.isEmpty()) {
                    JSONObject jsonObject = JSON.parseObject(body);
                    String cityIds = jsonObject.getString("cityIds");
                    if (StringUtils.isEmpty(cityIds) && null != citys && citys.containsKey("data_arr")) {
                        jsonObject.put("cityIds", citys.get("data_arr"));
                    }
                    String postIds = jsonObject.getString("postIds");
                    if (StringUtils.isEmpty(postIds) && null != posts && posts.containsKey("data_arr")) {
                        jsonObject.put("postIds", posts.get("data_arr"));
                    }
                    String stationIds = jsonObject.getString("stationIds");
                    if (StringUtils.isEmpty(stationIds) && null != stations && stations.containsKey("data_arr")) {
                        jsonObject.put("stationIds", stations.get("code_arr"));
                    }

                }
            }
            if (null == citys && null == posts && null == stations || (null != citys && citys.size() == 0 && null != posts && posts.size() == 0
                    && null != stations && stations.size() == 0)) {
                throw new PromiseException("您没有权限查看此报表");
            }

        }
        filterChain.doFilter(requestWrapper, res);


    }

    private void reflectSetHeader(HttpServletRequest request, String key, String value) {
        Class<? extends ServletRequest> requestClass = ((RequestWrapper) request).getRequest().getClass();
        try {
            Field requestField = requestClass.getDeclaredField("request");
            requestField.setAccessible(true);
            Object requestObj = requestField.get(((RequestWrapper) request).getRequest());
            Field coyoteRequestField = requestObj.getClass().getDeclaredField("coyoteRequest");
            coyoteRequestField.setAccessible(true);
            Object coyoteRequestObj = coyoteRequestField.get(requestObj);
            Field headersField = coyoteRequestObj.getClass().getDeclaredField("headers");
            headersField.setAccessible(true);
            MimeHeaders headersObj = (MimeHeaders) headersField.get(coyoteRequestObj);
            headersObj.removeHeader(key);
            headersObj.addValue(key).setString(value);
        } catch (Exception e) {
            System.out.println("设置请求头失败，失败原因：" + e.getMessage());
        }
    }

}
