package com.jzlife.report.listener;


import com.jzlife.report.config.SendSms;
import com.jzlife.report.schedule.ImportExcelTask;
import com.jzlife.report.util.email.EmailUtils;
import com.sun.mail.imap.IMAPFolder;
import com.sun.mail.util.MailSSLSocketFactory;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import javax.mail.*;
import javax.mail.event.MessageCountEvent;
import javax.mail.event.MessageCountListener;
import java.io.IOException;
import java.security.GeneralSecurityException;
import java.util.Properties;


@Service
@Slf4j
public class ALiEmailListener {


    private IMAPFolder imapFolder;

    private final String aliFromSuffix = "alibaba-inc.com>";
//    private final String aliFromSuffix = "lifejingzhi.com>"; //for test


    @Autowired
    private ImportExcelTask task;

    //开启监听 盒马不发邮件了
    @SneakyThrows
//    @PostConstruct
    public void init() {
        new Thread(() -> {
            try {
                new ALiEmailListener().startService();
            } catch (MessagingException e) {
                throw new RuntimeException(e);
            }
        }).start();
    }

    public void startService() throws MessagingException {
        try {
            setup();
        } catch (MessagingException e) {
            System.out.println("Error configuring imap server:");
            System.out.println(e);
            System.exit(1);
        } catch (GeneralSecurityException e) {
            e.printStackTrace();
        }


        imapFolder.addMessageCountListener(new MessageCountListener() {


            public void messagesAdded(MessageCountEvent arg0) {
                for (Message message : arg0.getMessages()) {
                    Address[] from;
                    try {
                        from = message.getFrom();
                    } catch (MessagingException e) {
                        throw new RuntimeException(e);
                    }
                    if (from != null) {
                        for (Address address : from) {
                            if (address.toString().endsWith(aliFromSuffix)) {
                                System.out.println("New message from " + address);
                                try {
                                    if (EmailUtils.isContainAttach(message)) {
                                        System.out.println("start to process email with attachment");
                                        Thread thread = getThread(message);
                                        thread.join();
                                        log.info("本地处理完毕，同步数据中~~");
                                        task.importReportData();
                                        log.info("调用report服务写入数据到数据库完毕");

                                    }
                                } catch (MessagingException | IOException e) {
                                    System.out.println(e.getMessage());
                                    throw new RuntimeException(e);
                                } catch (InterruptedException e) {
                                    throw new RuntimeException(e);
                                }
                            }
                        }
                    }
                }
            }

            @NotNull
            private Thread getThread(Message message) {
                Runnable a = () -> {
                    try {
                        EmailUtils.saveAttachMent(message);
                    } catch (MessagingException | IOException e) {
                        throw new RuntimeException(e);
                    }
                };
                Thread thread = new Thread(a);
                thread.start();
                return thread;
            }

            @Override
            public void messagesRemoved(MessageCountEvent arg0) {
                System.out.println("Message was removed.");
            }

        });

        for (; ; ) {
            try {
                imapFolder.idle();
            } catch (Exception e) {
                try {
                    Thread.sleep(1000 * 3);
                    startService();
                } catch (Exception ignored) {

                } finally {
                    SendSms.sendAliMs("18550658899");
                    imapFolder.idle();
                }
            }
        }
    }


    private void setup() throws MessagingException, GeneralSecurityException {
        Properties props = new Properties();
        MailSSLSocketFactory sf = new MailSSLSocketFactory();
        sf.setTrustAllHosts(true);
        props.setProperty("mail.store.protocol", "imap");
        props.setProperty("mail.imaps.ssl.enable", "true");
        props.setProperty("mail.imaps.connectiontimeout", "10000");
        props.setProperty("mail.imaps.timeout", "10000");
        props.put("mail.imaps.ssl.socketFactory", sf);
        Session session = Session.getDefaultInstance(props, null);
        Store store = session.getStore();

        store.connect("imap.qiye.aliyun.com", 143, "documents@lifejingzhi.com", "JZSHjzsh123456");
        imapFolder = (IMAPFolder) store.getFolder("INBOX");
        System.out.println("start listen email ~~~~");
        imapFolder.open(Folder.READ_WRITE);
    }

}
