package com.jzlife.report.service.hmorder.impl;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.github.pagehelper.PageHelper;
import com.jzlife.report.dao.hmorder.EtlGoalMapper;
import com.jzlife.report.dto.EtlGoalDto;
import com.jzlife.report.entity.hmorder.EtlGoal;
import com.jzlife.report.service.hmorder.SalesTargetValueService;
import com.jzlife.report.util.JsonUtil;
import com.jzlife.report.util.ResponseUtil;
import com.jzlife.report.util.page.BeanUtil;
import com.jzlife.report.util.page.PagedResult;

@Service
public class SalesTargetValueServiceImpl implements SalesTargetValueService {

	@Autowired
	EtlGoalMapper etlgoalMapper;

	@Override
	public Object insertTargetValue(List<EtlGoal> list, Long updateUser) {
		// 导入的数据，可能有脏数据，重复数据（数据库里的）
		// 要新增的，要修改的
		List<String> failList = new ArrayList<String>();

		int i = 0;
		for (EtlGoal e : list) {
			i++;
			if (StringUtils.isBlank(e.getBusinessType()) || StringUtils.isBlank(e.getCategory())
					|| StringUtils.isBlank(e.getCity()) || StringUtils.isBlank(e.getEndMonth())
					|| StringUtils.isBlank(e.getStartMonth()) || StringUtils.isBlank(e.getType())) {
				failList.add("第" + i + "行有空数据，请核查");
			}

			List<EtlGoal> templist = etlgoalMapper.selectBytype(e);
			if (templist.size() > 1) {
				failList.add("第" + i + "行数据存在重复，请核查");
			}

			
			if(e.getGoal()==null) {
				e.setGoal(new BigDecimal(0));
			}
			
			if (templist.size() == 1) {
				// 存在就更新
				e.setId(templist.get(0).getId());
				e.setUpdateTime(new Date());
				e.setUpdateUser(updateUser);
				e.setLastUpdateComment(JsonUtil.getJson(templist.get(0)));
				etlgoalMapper.updateByPrimaryKeySelective(e);
			}

			if (templist.size() == 0) {
				// 不存在就新增
				etlgoalMapper.insertSelective(e);
			}

		}

		if (failList.size() > 0) {
			return ResponseUtil.fail(failList);
		}

		return ResponseUtil.ok();

	}

	@Override
	public void updateTargetValue(EtlGoal e) {
		EtlGoal eg = etlgoalMapper.selectByPrimaryKey(e.getId());
		if (null != eg) {
			e.setUpdateTime(new Date());
			e.setLastUpdateComment(JsonUtil.getJson(eg));
			etlgoalMapper.updateByPrimaryKeySelective(e);
		}

	}

	@Override
	public void addTargetValue(EtlGoal e) {
		etlgoalMapper.insertSelective(e);

	}

	@Override
	public void deleteTargetValue(Integer id) {
		etlgoalMapper.deleteByPrimaryKey(id);
	}

	@Override
	public PagedResult<EtlGoal> pageDataList(EtlGoalDto dto) {
		Integer pageNo = dto.getPageNo() == null ? 1 : dto.getPageNo();
		Integer pageSize = dto.getPageSize() == null ? 10 : dto.getPageSize();
		// startPage是告诉拦截器说我要开始分页了。分页参数是这两个。
		PageHelper.startPage(pageNo, pageSize);

		EtlGoal eg = new EtlGoal();
		BeanUtils.copyProperties(dto, eg);
		List<EtlGoal> selectBytype = etlgoalMapper.selectBytype(eg);
		return BeanUtil.toPagedResult(selectBytype);
	}

}
