package com.jzlife.report.service.newreport.impl;

import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jzlife.report.dao.manager.EntryResignLossRecordMapper;
import com.jzlife.report.dao.manager.TempInvestInfoMapper;
import com.jzlife.report.dao.reports.ZSsettlementMapper;
import com.jzlife.report.dto.EntryResignLossRecord;
import com.jzlife.report.dto.export.BjSettlementExportDto;
import com.jzlife.report.dto.export.BmAgencyFeeExportDto;
import com.jzlife.report.dto.export.BmClueCostExportDto;
import com.jzlife.report.dto.export.BmSettlementExportDto;
import com.jzlife.report.dto.export.BmSupplyInfoExportDto;
import com.jzlife.report.service.newreport.InvestmentReportService;
import com.jzlife.report.util.ResponseUtil;
import com.jzlife.report.vo.DayReportBaseVo;
import com.jzlife.report.vo.InvestmentAndResignVo;
import com.jzlife.report.vo.InvestmentVo;
import com.jzlife.report.vo.LossAndResignVo;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import cn.afterturn.easypoi.excel.entity.enmus.ExcelType;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.util.*;
import java.util.stream.Collectors;


@Service
@Slf4j
public class InvestmentReportServiceImpl implements InvestmentReportService {


    @Resource
    private TempInvestInfoMapper investInfoMapper;

    @Resource
    private EntryResignLossRecordMapper resignLossRecordMapper;

    @Autowired
    private ZSsettlementMapper zsSettlementMapper;


    @Override
    public Map<String, Object> investmentAndResignList(InvestmentAndResignVo vo) {
        Map<String, Object> result = new HashMap<>();
        List<String> category = new ArrayList<>();
        // 入职情况
        Map<String, Long> map1 = investInfoMapper.selectAllSourceNoTrainInfoByCityOrStation(vo);
        Map<String, Long> realMap = new HashMap<>();
        if (StringUtils.isNotEmpty(vo.getResource())) {
            // 过滤 来源
            String[] resources = vo.getResource().split(",");
            for (String resource : resources) {
                String mean = transalationResource(resource);
                realMap.put(mean, map1.remove(mean));
            }
            realMap.entrySet().stream().sorted(Map.Entry.comparingByValue()).forEach(e -> category.add(e.getKey()));
        }else{
            map1.entrySet().stream().sorted(Map.Entry.comparingByValue()).forEach(e -> category.add(e.getKey()));
        }

        InvestmentVo investmentVo = new InvestmentVo();
        investmentVo.setName("未跟单人数");
        List<Long> noTran = new ArrayList<>();
        for (String s : category) {
            Long aLong = map1.get(s);
            noTran.add(aLong);
        }
        investmentVo.setData(noTran);


        Map<String, Object> map2 = investInfoMapper.selectAllSourceSignInfoByCityOrStation(vo);
        InvestmentVo investmentVo1 = new InvestmentVo();
        investmentVo1.setName("入职人数");
        List<Long> signs = new ArrayList<>();
        if (noTran.isEmpty() || noTran.stream().anyMatch(Objects::isNull)) {
            //优化排序 如果有未跟单 情况 使用未跟单 人数排序 没有则使用 入职人数排序
            category.sort( (a,b)->{
                Long aLong = (Long)map2.get(a);
                Long bLong = (Long)map2.get(b);
                return aLong.compareTo(bLong);
            });
        }
        for (String out : category) {
            Long aLong = (Long) map2.get(out);
            signs.add(aLong);
        }
        investmentVo1.setData(signs);
        List<InvestmentVo> investmentVos = Arrays.asList(investmentVo, investmentVo1);
        result.put("series", investmentVos);
        result.put("category", category);
        //流失率 当月解约 /（月初人数 + 当月入职） *100%
        //当月离职人数
        Map<String, Object> temp1 = investInfoMapper.selectByCityOrStation(vo);
        DecimalFormat decimalFormat = new DecimalFormat("##.00%");
        Long bjresignCount = (Long) temp1.get("保洁离职人数");
        Long plresignCount = (Long) temp1.get("品类离职人数");
        Long bjsignCount = (Long) temp1.get("保洁入职人数");
        Long plsignCount = (Long) temp1.get("品类入职人数");
        Map<String, Long> workingCounts = investInfoMapper.getWorkingCountByStartTimeAndCityOrStation(vo);
        Long bjCount = workingCounts.get("bjCount");
        Long plCount = workingCounts.get("plCount");
        if (plCount == 0L) {
            temp1.put("品类流失率", "NOData");
        } else if (plresignCount == 0L) {
            temp1.put("品类流失率", "0.00%");
        } else {
            temp1.put("品类流失率", decimalFormat.format(new BigDecimal(plresignCount).divide(new BigDecimal(plCount + plsignCount), 4, RoundingMode.HALF_UP)));
        }
        if (bjCount == 0L) {
            temp1.put("保洁流失率", "NOData");
        } else if (bjresignCount == 0L) {
            temp1.put("保洁流失率", "0.00%");
        } else {
            temp1.put("保洁流失率", decimalFormat.format(new BigDecimal(bjresignCount).divide(new BigDecimal(bjCount + bjsignCount), 4, RoundingMode.HALF_UP)));
        }
        result.put("conversion", temp1);
        return result;
    }

    private String transalationResource(String resource) {
        switch (resource) {
            case "1":
                return "渠道";
            case "2":
                return "合伙人";
            case "3":
                return "自招";
            case "4":
                return "转介绍";
            case "5":
                return "复职";
            case "6":
                return "自然流量";

        }
        return "";
    }

    @Override
    public Object getResignAndLossReason(String type, String startTime, String endTime, String city, String postStation, String serviceStation) {
        List<EntryResignLossRecord> entryResignLossRecords;
        if (StringUtils.equals(type, "1,2") || StringUtils.isEmpty(type)) {
            entryResignLossRecords = resignLossRecordMapper.selectAll(startTime, endTime,city,postStation,serviceStation);
        } else {
            entryResignLossRecords = resignLossRecordMapper.selectByType(type, startTime, endTime,city,postStation,serviceStation);
        }
        JSONObject jsonObject = new JSONObject();
        Map<String, Integer> hash1 = new HashMap<>();
        Map<String, Integer> hash2 = new HashMap<>();
        Map<String, List<EntryResignLossRecord>> collect1 = entryResignLossRecords.stream()
                .filter(a -> a.getType() == 1).collect(Collectors.groupingBy(EntryResignLossRecord::getLevelTwo));
        Map<String, List<EntryResignLossRecord>> collect2 = entryResignLossRecords.stream()
                .filter(a -> a.getType() == 2).collect(Collectors.groupingBy(EntryResignLossRecord::getLevelOne));
        collect1.forEach((a, b) -> hash1.put(a, collect1.get(a).size()));
        collect2.forEach((a, b) -> hash2.put(a, collect2.get(a).size()));
        if (StringUtils.equals(type, "1,2") || StringUtils.isEmpty(type)) {
            LossAndResignVo vo = new LossAndResignVo();
            vo.setType(2);
            vo.setCountMap(tranMap(hash2));
            jsonObject.put("loss", vo);
            LossAndResignVo vo1 = new LossAndResignVo();
            vo1.setType(1);
            vo1.setCountMap(tranMap(hash1));
            jsonObject.put("resign", vo1);
        } else {
            if (StringUtils.equals(type, "2")) {
                LossAndResignVo vo = new LossAndResignVo();
                vo.setType(2);
                vo.setCountMap(tranMap(hash2));
                jsonObject.put("loss", vo);
            } else if (StringUtils.equals(type, "1")) {
                LossAndResignVo vo1 = new LossAndResignVo();
                vo1.setType(1);
                vo1.setCountMap(tranMap(hash1));
                jsonObject.put("resign", vo1);
            }
        }
        return jsonObject;
    }

    @Override
    public PageInfo<Map<String, Object>> dayInvestMentDetail(DayReportBaseVo vo) {
        PageHelper.startPage(vo.getStartPage(), vo.getLimit());
        List<Map<String, Object>> mapList = investInfoMapper.dayInvestMentDetail(vo);
        return new PageInfo<>(mapList);
    }

    @Override
    public List<Map<String, Object>> dayInvestMentDetailExport(DayReportBaseVo vo) {
        return investInfoMapper.dayInvestMentDetail(vo);
    }


    private Map<String, Integer> tranMap(Map<String, Integer> result2) {
        Map<String, Integer> temp = new HashMap<>();
        for (String key : result2.keySet()) {
            String means = resignLossRecordMapper.getMeansByValue(key);
            temp.put(means, result2.get(key));
        }
        LinkedHashMap<String, Integer> result = new LinkedHashMap<>(temp);
        LinkedHashMap<String, Integer> abc = new LinkedHashMap<>();

        result.entrySet().stream().sorted(Comparator.comparingInt(Map.Entry::getValue)).forEach(a -> abc.put(a.getKey(), a.getValue()));

        return abc;
    }

	@Override
	public Object bjSettlement(Integer pageNo, Integer pageSize, Boolean isExport, String ofMonth, String store,
			String workerName) {
		if(isExport) {
			return zsSettlementMapper.getBJSettlement(ofMonth, store, workerName);
		}

		PageHelper.startPage(pageNo,pageSize);
		List<BjSettlementExportDto> list = zsSettlementMapper.getBJSettlement(ofMonth, store, workerName);

		return ResponseUtil.ok(new PageInfo<BjSettlementExportDto>(list));
	}

	@Override
	public Object bmSettlement(String ofMonth,Boolean isExport,HttpServletResponse response) {
		if (isExport) {
			//导出 #保姆结算,保姆供给明细,线索成本,中介费
			List<BmSettlementExportDto> bmSettlement = zsSettlementMapper.getBmSettlement(ofMonth.replaceAll("-", ""));
			List<BmSupplyInfoExportDto> bmSupplyInfo = zsSettlementMapper.getBmSupplyInfo(ofMonth, "精楷家政");
			List<BmClueCostExportDto> bmClueCost = zsSettlementMapper.getBmClueCost(ofMonth);
			List<BmAgencyFeeExportDto> bmAgencyFee = zsSettlementMapper.getBmAgencyFee(ofMonth);

//    		if (list != null && list.size() > 0) {
//                ExcelUtils.exportExcel(list, null, "Sheet1", BjSettlementExportDto.class, "保洁结算.xls", response);
//            }

            List<Map<String, Object>> exportParamList = new ArrayList<>();

			ExportParams params = new ExportParams("保姆结算", "保姆结算", ExcelType.XSSF);
	        Map<String, Object> valueMap1 = new HashMap<String, Object>();
	        valueMap1.put("title", params);
	        valueMap1.put("data", bmSettlement);
	        valueMap1.put("entity", BmSettlementExportDto.class);
	        exportParamList.add(valueMap1);

	        ExportParams params2 = new ExportParams("保姆供给明细", "保姆供给明细", ExcelType.XSSF);
	        Map<String, Object> valueMap2 = new HashMap<String, Object>();
	        valueMap2.put("title", params2);
	        valueMap2.put("data", bmSupplyInfo);
	        valueMap2.put("entity", BmSupplyInfoExportDto.class);
	        exportParamList.add(valueMap2);

			ExportParams params3 = new ExportParams("线索成本", "线索成本", ExcelType.XSSF);
	        Map<String, Object> valueMap3 = new HashMap<String, Object>();
	        valueMap3.put("title", params3);
	        valueMap3.put("data", bmClueCost);
	        valueMap3.put("entity", BmClueCostExportDto.class);
	        exportParamList.add(valueMap3);

	        ExportParams params4 = new ExportParams("中介费", "中介费", ExcelType.XSSF);
	        Map<String, Object> valueMap4 = new HashMap<String, Object>();
	        valueMap4.put("title", params4);
	        valueMap4.put("data", bmAgencyFee);
	        valueMap4.put("entity", BmAgencyFeeExportDto.class);
	        exportParamList.add(valueMap4);

	        try {
	            Workbook workbook = ExcelExportUtil.exportExcel(exportParamList, ExcelType.XSSF);
	            response.setContentType("application/xlsx;charset=utf-8");
	            response.setHeader("Content-Disposition", "attachment; filename=" + System.currentTimeMillis()+".xlsx");
	            workbook.write(response.getOutputStream());
	        }catch (Exception e){
	           e.printStackTrace();
	        }
	        return null;
    	}

		List<BmSettlementExportDto> bmSettlement = zsSettlementMapper.getBmSettlement(ofMonth.replaceAll("-", ""));

		return ResponseUtil.ok(bmSettlement);
	}


}
