package com.jzlife.report.service.report.impl;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.stream.Collectors;

import org.apache.commons.compress.utils.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.jzlife.report.dao.reports.BigScreeMapper;
import com.jzlife.report.dto.EchartsLine;
import com.jzlife.report.dto.dto.ChannelMontOrDayGmv;
import com.jzlife.report.dto.dto.OrderLastInfo;
import com.jzlife.report.dto.dto.YearVerifyAmount;
import com.jzlife.report.service.report.BigScreeReportService;
import com.jzlife.report.util.DateUtil;
import com.jzlife.report.util.DateUtils;
import com.jzlife.report.util.ResponseUtil;

import javax.annotation.Resource;

@Service
public class BigScreeReportServiceImpl implements BigScreeReportService {

	@Resource
	BigScreeMapper bigScreeMapper;

	//改成月派单量
	public Object yearServiceGMV(String date, Integer type) {
		Date stringToDate = DateUtil.stringToDate(date);

		Map<String, Object> data = new HashMap<String, Object>();
		List<EchartsLine> echLine = new ArrayList<EchartsLine>();
		EchartsLine line = new EchartsLine("bar");
		line.setName("近6个月");

		//月份
		List<String> month = new ArrayList<String>();
		for (int i = 11; i >= 0; i--) {// 近12个月
			Date addMonth = DateUtil.addMonth(stringToDate, -i);
			month.add(DateUtil.dateToString(addMonth, "yyyy-MM"));
		}
		data.put("days", month);

		List<YearVerifyAmount> selectMonth = bigScreeMapper.selectBookedByMonth(Lists.newArrayList());
		List<Object> collect = selectMonth.stream().filter(e -> month.contains(e.getMonth())).map(e -> e.getAmount()).collect(Collectors.toList());
		line.setData(collect);
		echLine.add(line);

		data.put("echLiss", echLine);
		return ResponseUtil.ok(data);
	}

	@Override
	public Object salesGmvAndBookcount(String date) {
		return ResponseUtil.ok(mapData(date));
	}

	@Override
	public Map<String, Object> mapData(String date) {
		// 实时销售明细最近20条后台，非当日随机取对应天20条
		Boolean iscurrDay = DateUtil.getCurrentDate().equals(date);// 是当天

		//滚动销售数据
		String st = iscurrDay ? (date + " 00:00:00") : (date + " 21:30:12");
		List<OrderLastInfo> orderSalesInfo = bigScreeMapper.selectOrderLastInfo(iscurrDay, 1, st, date + " 23:59:59");

		// 省份销售单量（地图展示）
		List<Map<String, Object>> countProvinceSalesNumber = bigScreeMapper.countProvinceSalesNumber(1);

		//最新30秒销售
		List<Map<String, Object>> newProvinceSalesNumber = null;//bigScreeMapper.countProvinceSalesNumber(0);

		Map<String, Object> data = new HashMap<String, Object>();
		data.put("orderSalesInfo", orderSalesInfo);
		data.put("countProvinceSalesNumber", countProvinceSalesNumber);
		data.put("newProvinceSalesNumber", newProvinceSalesNumber == null ? new ArrayList<>() : newProvinceSalesNumber);

		data.put("nowDate", iscurrDay ? DateUtil.getCurrentDateTime() : date + " 23:59:59");
		data.put("iscurrDay", iscurrDay);
		return data;
	}

	//日销售汇总,月派单,月销售汇总,渠道销售汇总
	@Override
	public Object monthGmv(String date) {

		String month = date.substring(0, date.lastIndexOf("-"));
		BigDecimal sumMonthSalesGMV = new BigDecimal("0");//月累计销售
		BigDecimal sumMonthVerifyGMV = new BigDecimal("0");//月累计核销
		Integer sumMonthOrders = 0;//月订单量

		// 日总销售额gmv
		BigDecimal sumDaySalesGMV = bigScreeMapper.sumPayOrder(date + " 00:00:00", date + " 23:59:59");

		//月
		List<ChannelMontOrDayGmv> monthSales = bigScreeMapper.sumMonthOrDaySales(2, month, date + " 00:00:00",
				date + " 23:59:59");
		// 天
		List<ChannelMontOrDayGmv> daySales = bigScreeMapper.sumMonthOrDaySales(1, month, date + " 00:00:00",
				date + " 23:59:59");
		//月核销
		List<ChannelMontOrDayGmv> monthVerifyGmv = bigScreeMapper.monthVerifyGmv(date.substring(0, 7));

		for (ChannelMontOrDayGmv g : monthSales) {
			sumMonthSalesGMV = sumMonthSalesGMV.add(g.getAmount());
			sumMonthOrders += g.getSalesNumber();
			daySales.forEach(e -> {
				if (g.getChannel().equals(e.getChannel())) {
					e.setMonthSalesGMV(g.getAmount());
					e.setMonthOrders(g.getSalesNumber());
				}
			});
		}

		for (ChannelMontOrDayGmv g : monthVerifyGmv) {
			sumMonthVerifyGMV = g.getAmount().add(sumMonthVerifyGMV);
			daySales.forEach(e -> {
				if (g.getChannel().equals(e.getChannel())) {
					e.setMonthVerifyGmv(g.getAmount());
					BigDecimal divide = g.getAmount().multiply(new BigDecimal(100)).divide(e.getMonthSalesGMV(), 2, BigDecimal.ROUND_CEILING).setScale(2, BigDecimal.ROUND_HALF_UP);
					e.setMonthVerifyRate(divide + "%");
				}
			});
		}


		Map<String, Object> data = new HashMap<String, Object>();
		data.put("month", month.substring(month.indexOf("-") + 1, month.length()).replace("0", ""));
		data.put("monthSalesGMV", sumMonthSalesGMV);
		data.put("monthVerifyGMV", sumMonthVerifyGMV);
		data.put("monthOrders", sumMonthOrders);

		//获取月派单量
		ChannelMontOrDayGmv provided = bigScreeMapper.sumMonthProvided();
		data.put("monthProvided", provided == null ? 0 : provided.getMonthProvided());
		data.put("monthPlatformProvided", provided == null ? 0 : provided.getMonthPlatformProvided());

		//当日销售汇总
		data.put("sumDaySalesGMV", sumDaySalesGMV);
		data.put("moreHistory", false);//是否新纪录

		//设置渠道图标和渠道销售数据
		setIcon(daySales);
		data.put("channelData", daySales);

		//map数据和滚动销售数据
		data.putAll(mapData(date));

		return ResponseUtil.ok(data);
	}


	public void setIcon(List<ChannelMontOrDayGmv> list) {
		// 微信商城
		// 抖音生活
		// 快手
		// 美团点评
		// 盒马鲜生
		list.forEach(e -> {
			if (e.getChannel().equals("微信商城")) {
				e.setIconA("https://servant.lifejingzhi.com/file/jzreport/bigicon/weixin.png");
				e.setIconB("https://servant.lifejingzhi.com/file/jzreport/bigicon/wx.png");
			}
			if (e.getChannel().equals("抖音生活")) {
				e.setIconA("https://servant.lifejingzhi.com/file/jzreport/bigicon/douyin.png");
				e.setIconB("https://servant.lifejingzhi.com/file/jzreport/bigicon/dy.png");
			}
			if (e.getChannel().equals("快手")) {
				e.setIconA("https://servant.lifejingzhi.com/file/jzreport/bigicon/kuaishou.png");
				e.setIconB("https://servant.lifejingzhi.com/file/jzreport/bigicon/ks.png");
			}
			if (e.getChannel().equals("美团点评")) {
				e.setIconA("https://servant.lifejingzhi.com/file/jzreport/bigicon/meituan.png");
				e.setIconB("https://servant.lifejingzhi.com/file/jzreport/bigicon/mt.png");
			}
//			if(e.getChannel().equals("盒马鲜生")) {
//				e.setIconA("https://servant.lifejingzhi.com/file/jzreport/bigicon/hema.png");
//				e.setIconB("https://servant.lifejingzhi.com/file/jzreport/bigicon/hm.png");
//			}
		});
	}

}
