package com.jzlife.report.service.report.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jzlife.report.dao.reports.BjManagerToolMapper;
import com.jzlife.report.entity.reports.*;
import com.jzlife.report.service.report.BjManagerToolService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class BjManagerToolServiceImpl implements BjManagerToolService {

    @Autowired
    private BjManagerToolMapper bjManagerToolMapper;

    /**
     * 可邀约好评
     *
     * @param citys   城市
     * @param month   月份
     * @param station 服务站
     **/
    @Override
    public Map<String, Object> evaluationInfo(String citys, String month, String station) {
        List<EvaluationInfoResult> evaluationInfo = bjManagerToolMapper.evaluationInfo(citys, month, station);
        Map<String, Object> result = new HashMap<>();
        result.put("evaluationInfo", evaluationInfo); // 服务站汇总信息
        return result;
    }

    @Override
    public Map<String, Object> evaluationInfoByStation(String citys, String month, String station, Integer pageNum, Integer pageSize) {
        Map<String, Object> result = new HashMap<>();
        //服务站每日信息
        PageHelper.startPage(pageNum, pageSize);
        List<EvaluationInfoDetailResult> detail = bjManagerToolMapper.evaluationInfoDetail(citys, month, station);
        PageInfo<EvaluationInfoDetailResult> resultPageInfo = new PageInfo<>(detail);
        result.put("detail", resultPageInfo);
        return result;
    }

    @Override
    public Map<String, Object> evaluationInfoByPearson(String citys, String month, String station, Integer pageNum, Integer pageSize) {
        Map<String, Object> result = new HashMap<>();
        PageHelper.startPage(pageNum, pageSize);
        //服务者每日信息
        List<EvaluationInfoDetailByPersonResult> personDetails = bjManagerToolMapper.evaluationInfoDetailByPerson(citys, month, station);
        PageInfo<EvaluationInfoDetailByPersonResult> resultPageInfo = new PageInfo<>(personDetails);
        result.put("detail", resultPageInfo);
        return result;
    }

    @Override
    public Map<String, Object> individualRanking(String citys, String month, String workerName, Integer pageNum, Integer pageSize, String stations,String goodRank) {
        Map<String, Object> result = new HashMap<>();
        PageHelper.startPage(pageNum, pageSize);
        List<IndividualRankingResult> individualRankingResults = bjManagerToolMapper.individualRanking(citys, month, workerName,stations,goodRank);
        PageInfo<IndividualRankingResult> resultPageInfo = new PageInfo<>(individualRankingResults);
        result.put("detail", resultPageInfo);
        return result;
    }

    @Override
    public Map<String, Object> individualRankingDetail(String month, String workerName, Integer pageNum, Integer pageSize, String citys, String stations) {
        Map<String, Object> result = new HashMap<>();
        PageHelper.startPage(pageNum, pageSize);
        List<IndividualRankingDetailResult> individualRankingResults = bjManagerToolMapper.individualRankingDetail(month, workerName,citys,stations);
        PageInfo<IndividualRankingDetailResult> resultPageInfo = new PageInfo<>(individualRankingResults);
        result.put("detail", resultPageInfo);
        return result;
    }

    @Override
    public Map<String, Object> signAdnClockInfo(String citys, String month) {
        Map<String, Object> result = new HashMap<>();
        List<SignAdnClockInfoResult> signAdnClockInfo = bjManagerToolMapper.signAdnClockInfo(citys, month);
        result.put("signAdnClockInfo", signAdnClockInfo);
        result.put("month", month);
        return result;
    }

    @Override
    public Map<String, Object> signAdnClockInfoByStation(String citys, String month, String station) {
        Map<String, Object> result = new HashMap<>();
        List<SignAdnClockInfoByStationResult> signAdnClockInfoByStation = bjManagerToolMapper.signAdnClockInfoByStation(citys, month, station);
        result.put("signAdnClockInfoByStation", signAdnClockInfoByStation);
        if (StringUtils.isNoneBlank(station)) {
            result.put("station", station);
        }
        result.put("month", month);
        return result;
    }

    @Override
    public Map<String, Object> signAdnClockOnDoorAndLeaveDoorInfo(String citys, String month, String station, String workerName,
                                                                  String queryType, Integer pageNum, Integer pageSize,String onOrLeaveDoorType) {
        Map<String, Object> result = new HashMap<>();
        PageHelper.startPage(pageNum, pageSize);
        if (StringUtils.isNoneBlank(station)) {
            result.put("station", station);
        }
        result.put("month", month);
        if (StringUtils.isNoneBlank(workerName)) {
            result.put("workerName", workerName);
        }
        if (queryType.equals("1")) {
            //到门情况
            List<SignAdnClockOnDoorInfoResult> ondoorInfo = bjManagerToolMapper.signAdnClockOnDoorInfo(citys, month, station, workerName,onOrLeaveDoorType);
            result.put("ondoorInfo", new PageInfo<>(ondoorInfo));
            return result;
        } else if (queryType.equals("2")) {
            //离门状况
            List<SignAdnClockOnDoorInfoResult> leaveDoorInfo = bjManagerToolMapper.signAdnClockLeaveDoorInfo(citys, month, station, workerName,onOrLeaveDoorType);
            result.put("leaveDoorInfo", new PageInfo<>(leaveDoorInfo));
        }
        return result;

    }

    @Override
    public List<SignAdnClockOnDoorInfoResult> signAdnClockOnDoorAndLeaveDoorInfoNoPage(String citys, String month, String station, String workerName, String queryType,String onOrLeaveDoorType) {
        if (queryType.equals("1")) {
            //到门情况
            return bjManagerToolMapper.signAdnClockOnDoorInfo(citys, month, station, workerName,onOrLeaveDoorType);

        } else if (queryType.equals("2")) {
            //离门状况
            return bjManagerToolMapper.signAdnClockLeaveDoorInfo(citys, month, station, workerName,onOrLeaveDoorType);
        } else {
            return new ArrayList<>();
        }
    }

    @Override
    public List<EvaluationInfoDetailResult> evaluationInfoByStationNoPage(String citys, String month, String station, Integer pageNum, Integer pageSize) {
        return bjManagerToolMapper.evaluationInfoDetail(citys, month, station);
    }

    @Override
    public List<EvaluationInfoDetailByPersonResult> evaluationInfoByPearsonNoPage(String citys, String month, String station, Integer pageNum, Integer pageSize) {
        return bjManagerToolMapper.evaluationInfoDetailByPerson(citys, month, station);
    }

    @Override
    public List<IndividualRankingDetailResult> individualRankingDetailNoPage(String month, String workerName, String citys, String stations) {
        return bjManagerToolMapper.individualRankingDetail(month, workerName, citys, stations);
    }

    @Override
    public List<IndividualRankingResult> individualRankingNoPage(String citys, String month, String workerName, Integer pageNum, Integer pageSize, String stations,String goodRank) {
        return  bjManagerToolMapper.individualRanking(citys, month, workerName, stations,goodRank);
    }
}
