package com.jzlife.report.service.report.impl;

import com.alibaba.fastjson.JSON;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jzlife.report.dao.reports.EtlTradeOrderBoardMapper;
import com.jzlife.report.dto.export.SelfRebuyDetailDTO;
import com.jzlife.report.exception.BusinessException;
import com.jzlife.report.service.ReBuy2Service;
import com.jzlife.report.util.MD5;
import com.jzlife.report.util.Task;
import com.jzlife.report.util.ThreadUtil;
import com.jzlife.report.vo.ReBuyDetailVo;
import com.jzlife.report.vo.ReBuyQueryVo;
import com.jzlife.report.vo.SelfReBuyQueryVo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

@Service
public class ReBuy2ServiceImpl implements ReBuy2Service {


    @Autowired
    private EtlTradeOrderBoardMapper boardMapper;

    //注入redisTemplate
    @Autowired
    private RedisTemplate redisTemplate;


    @Override
    public synchronized Map<String, Object> reBuyLine(ReBuyQueryVo vo) {
        //1. 城市、驿站、服务站维度 折线图
        if (StringUtils.isNotBlank(vo.getWorkerNo())) {
            Map<String, Object> s = boardMapper.getWorkerInfo(vo.getWorkerNo());
            vo.setCityIds(s.get("cityIds").toString());
            vo.setStationIds(s.get("stationIds").toString());
            vo.setPostIds(s.get("postIds").toString());
        }
        if (StringUtils.isNotBlank(vo.getCityIds())) {
            String cityIds = Arrays.stream(vo.getCityIds().split(",")).sorted(Comparator.comparingInt(Integer::parseInt)).collect(Collectors.joining(","));
            vo.setCityIds(cityIds);
        }
        if (StringUtils.isNotBlank(vo.getPostIds())) {
            String posts = Arrays.stream(vo.getPostIds().split(",")).sorted().collect(Collectors.joining(","));
            vo.setPostIds(posts);
        }
        if (StringUtils.isNotBlank(vo.getStationIds())) {
            String s = Arrays.stream(vo.getStationIds().split(",")).sorted().collect(Collectors.joining(","));
            vo.setStationIds(s);
        }
        //计算 vo中的startTime 对应的当周的周一
        String startTime = vo.getStartTime();
        LocalDate localDate = LocalDate.parse(startTime);
        int dayOfWeek = localDate.getDayOfWeek().getValue();
        LocalDate monday = localDate.minusDays(dayOfWeek - 1);
        ReBuyQueryVo cacheVo = new ReBuyQueryVo();
        cacheVo.setStartTime(monday.toString());
        cacheVo.setCityIds(vo.getCityIds());
        cacheVo.setPostIds(vo.getPostIds());
        cacheVo.setStationIds(vo.getStationIds());
        cacheVo.setWorkerNo(vo.getWorkerNo());
        String encryption = MD5.encryption(JSON.toJSONString(cacheVo));

        String query = "reBuyLine:" + encryption;
        String o = (String) redisTemplate.opsForValue().get(query);
        if (StringUtils.isNotBlank(o)) {
            //刷新缓存时间
            redisTemplate.expire(query, 3, TimeUnit.HOURS);
            return JSON.parseObject(o, Map.class);
        }


        List<Map<String, Object>> maps = boardMapper.callReBuyLine(vo);

        Map<String, Object> result = new HashMap<>();
        Map<String, Object> stringObjectMap = maps.stream().filter(m -> (Integer) m.get("is_split_city") == 0).findFirst().orElseThrow(() -> new BusinessException("没有数据"));
        result.put("index", stringObjectMap); //指标数据
        // 新增个人维度返回 总人数和复购数
        if (StringUtils.isNotBlank(vo.getWorkerNo())) {
            List<Map<String, Object>> data2Person = new ArrayList<>();
            maps.stream().filter(m -> (Integer) m.get("is_split_city") == 1).forEach(x -> {
                Map<String, Object> map = new HashMap<>();
                map.put("weekDesc", x.get("week_desc"));
                map.put("singleTotal", x.get("single_count"));
                map.put("singleRebuy", x.get("single_t7_transfer_cycle"));
                map.put("cycleRebuy", x.get("cycle_last_t30_fulfillment_count"));
                map.put("cycleTotal", x.get("cycle_last_fulfillment_count"));
                data2Person.add(map);
            });
            result.put("data2Person", data2Person);
        }


        Map<String, List<Map<String, Object>>> collect = maps.stream().filter(m -> (Integer) m.get("is_split_city") == 1)
                .collect(Collectors.groupingBy(n -> (String) n.get("city_ids")));
        String cityIds = vo.getCityIds();
        List<String> citys = new ArrayList<>();
        for (
                String s : cityIds.split(",")) {
            String cityName = boardMapper.getCityByCityId(s);
            if (StringUtils.isEmpty(cityName)) {
                citys.add("全国");
            } else {
                citys.add(cityName);
            }
        }
        result.put("citys", citys);
        if (StringUtils.isEmpty(cityIds) || cityIds.split(",").length == 1) {
            if (StringUtils.isEmpty(cityIds) || "0".equals(cityIds)) {
                cityIds = "-1";
            }
            List<BigDecimal> single_t7_transfer_cycle_rate = collect.get(cityIds).stream().map(a -> (String) a.get("single_t7_transfer_cycle_rate"))
                    .map(x -> BigDecimal.valueOf(Double.parseDouble(x)).setScale(4, RoundingMode.HALF_UP)).collect(Collectors.toList());//单次新客 7天内转周期率
            result.put("single_t7_transfer_cycle_rate", single_t7_transfer_cycle_rate); //单次折线图
            List<BigDecimal> cycle_rebuy_rate = collect.get(cityIds).stream().map(m -> (String) m.get("cycle_rebuy_rate")).map(x -> BigDecimal.valueOf(Double.parseDouble(x)).setScale(4, RoundingMode.HALF_UP)).collect(Collectors.toList());
            result.put("cycle_rebuy_rate", cycle_rebuy_rate); //周期折线图
        } else {

            List<List<BigDecimal>> line1 = new ArrayList<>();
            List<List<BigDecimal>> line2 = new ArrayList<>();
            for (String city : vo.getCityIds().split(",")) {
                List<BigDecimal> single_t7_transfer_cycle_rate = collect.get(city).stream().map(a -> (String) a.get("single_t7_transfer_cycle_rate"))
                        .map(x -> BigDecimal.valueOf(Double.parseDouble(x)).setScale(4, RoundingMode.HALF_UP)).collect(Collectors.toList());//单次新客 7天内转周期率
                line1.add(single_t7_transfer_cycle_rate);
                List<BigDecimal> cycle_rebuy_rate = collect.get(city).stream().map(m -> (String) m.get("cycle_rebuy_rate")).map(x -> BigDecimal.valueOf(Double.parseDouble(x)).setScale(4, RoundingMode.HALF_UP)).collect(Collectors.toList());
                line2.add(cycle_rebuy_rate);
            }
            result.put("single_t7_transfer_cycle_rate", line1); //单次折线图
            result.put("cycle_rebuy_rate", line2); //周期折线图
        }

        String a = boardMapper.getSingleWarnLine();
        result.put("single_t7_warn_line", a); //履约t+7转周期率预警线

        String b = boardMapper.getSingleNoStandardLine();
        result.put("single_rebuy_no_standard_warn_line", b); //单次新客7天内转周期率不达标预警线


        String c = boardMapper.getCycleWarnLine();
        result.put("cycle_rebuy_warn_line", c); //周期续约预警线

        String d = boardMapper.getCycleNoStandardLine();
        result.put("cycle_rebuy_no_standard_warn_line", d); //周期复购不达标预警线

        String e = boardMapper.getCrossWarnLine();
        result.put("cross_rebuy_warn_line", e); //交叉复购预警线

        List<String> weekDesc = maps.stream().filter(m -> (Integer) m.get("is_split_city") == 1).map(h -> (String) h.get("week_desc")).map(k -> k.startsWith("0") ? k.substring(1) : k)
                .distinct().collect(Collectors.toList());
        result.put("week_desc", weekDesc);

        redisTemplate.opsForValue().set(query, JSON.toJSONString(result), 1, TimeUnit.HOURS);
        return result;
    }

    @Override
    public Object reBuyDetail(ReBuyDetailVo vo) {
        if (vo.getPageNo() == null) {
            vo.setPageNo(1);
        }
        if (vo.getPageSize() == null) {
            vo.setPageSize(20);
        }
        if (vo.getIsExport() == null) {
            vo.setIsExport(false);
        }
        if (StringUtils.isNotBlank(vo.getCityIds()) && vo.getCityIds().equals("0")) {
            vo.setCityIds(null);
        }
        //将vo的开始时间 +7天 设置到vo的结束时间
        LocalDate start = LocalDate.parse(vo.getStartTime());
        // start 转成localdatetime并将开始时间重置成当周周一
        int value = start.atStartOfDay().getDayOfWeek().getValue();
        start = start.minusDays(value - 1);
        vo.setStartTime(start.toString());
        LocalDate end = start.plusDays(6);
        vo.setEndTime(end.toString());

        if (null == vo.getShowType() || vo.getShowType() == 1) {
            if (!vo.getIsExport()) {
                PageHelper.startPage(vo.getPageNo(), vo.getPageSize());
                List<Map<String, Object>> result = boardMapper.queryDetailByParam(vo);
                return new PageInfo<>(result);
            } else {
                return boardMapper.queryDetailByParam(vo);
            }
        } else {
            if (vo.getOrignGoodType() == null) {
                throw new BusinessException("服务者维度必须选择订单类型");
            }
            if (!vo.getIsExport()) {
                PageHelper.startPage(vo.getPageNo(), vo.getPageSize());
                List<Map<String, Object>> result = boardMapper.queryDetailByPerson(vo);
                return new PageInfo<>(result);
            } else {
                return boardMapper.queryDetailByPerson(vo);
            }
        }


    }

    @Override
    public void cleanRedisCacheFun() {
        //删除 redis中前缀为   "reBuyLine:"  的key
        Set<String> keys = redisTemplate.keys("reBuyLine:*");
        redisTemplate.delete(keys);
    }

    @Override
    public List<Map<String, Object>> bookTradeOrderHistory(String bookTradeOrder) {
        return boardMapper.bookTradeOrderHistory(bookTradeOrder);
    }

    @Override
    public Map<String, Object> selfReBuyLine(SelfReBuyQueryVo vo) throws ExecutionException, InterruptedException {
        Map<String, Object> result = new HashMap<>();
        processParam(vo);
        // 获取所有城市每个月份的复购率和 整体的复购金额、复购率和复购单量、单次整体复购金额、续约率、单次转周期律、单次整体复购率、单次

        List<String> months = new ArrayList<>();
        String month = vo.getMonth();
        if (StringUtils.isBlank(month)) {
            month = LocalDate.now().minusMonths(1).format(DateTimeFormatter.ofPattern("yyyy-MM"));
        }
        months.add(month);
        for (int i = 0; i < 5; i++) {
            month = LocalDate.parse(month + "-01").minusMonths(1).format(DateTimeFormatter.ofPattern("yyyy-MM"));
            months.add(month);
        }
        Collections.reverse(months);
        result.put("months", months);
        List<Integer> type = new ArrayList<>();
        type.add(1);
        type.add(2);
        type.add(3);
        Future<?> index = ThreadUtil.addTask(getTotalInfo(months, vo.getCityList(), vo.getFraName()));
        result.put("index", index.get());
        for (Integer i : type) {
            Future<?> future = ThreadUtil.addTask(getEveryMonth(months, vo.getCityList(), i, vo.getFraName()));
            if (i == 1) {
                result.put("single", future.get());
            } else if (i == 2) {
                result.put("cycle", future.get());
            } else {
                result.put("all", future.get());
            }
        }
        return result;
    }

    @Override
    public Object getFraList() {
        return boardMapper.getFraList();
    }

    @Override
    public PageInfo<SelfRebuyDetailDTO> selfReBuyDetailPage(SelfReBuyQueryVo vo) {
        processParam(vo);
        PageHelper.startPage(vo.getPageNo(), vo.getPageSize());
        List<SelfRebuyDetailDTO> list = boardMapper.selfReBuyDetail(vo);
        return new PageInfo<>(list);
    }

    private static void processParam(SelfReBuyQueryVo vo) {
        if (StringUtils.isNotBlank(vo.getCityList())) {
            String[] split = vo.getCityList().split(",");
            StringBuilder builder = new StringBuilder();
            for (String s : split) {
                if (!s.endsWith("市")&&!s.equals("全国")) {
                    s = s.concat("市");
                }
                builder.append(s).append(",");
            }
            vo.setCityList(builder.substring(0, builder.length() - 1));
        } else {
            vo.setCityList("全国");
        }
    }

    @Override
    public List<SelfRebuyDetailDTO> selfReBuyDetailNoPage(SelfReBuyQueryVo vo) {
        processParam(vo);
        return boardMapper.selfReBuyDetail(vo);
    }

    private Task getTotalInfo(List<String> month, String cityList, String fraName) {
        return new Task() {
            @Override
            public Object call() {
                return boardMapper.queryTotalData(month, cityList, fraName);
            }
        };
    }

    private Task getEveryMonth(List<String> months, String cityList, Integer type, String fraName) {
        return new Task() {
            @Override
            public Object call() {
                List<Map<String, Object>> totalOrderCountList = boardMapper.queryEveryCityMonthTotalOrder(months, cityList, type, fraName);
                List<Map<String, Object>> rebuyOrderCountList = boardMapper.queryEveryCityMonthRebuyOrder(months, cityList, type, fraName);
                //获取每个城市的复购率
                List<Map<String, List<String>>> result = new ArrayList<>();
                for (String city : cityList.split(",")) {
                    Map<String, List<String>> cityRebuyRateMap = new HashMap<>();
                    List<String> cityRebuyRate = new ArrayList<>(Arrays.asList("0.0000", "0.0000", "0.0000", "0.0000", "0.0000", "0.0000"));
                    for (int i = 0; i < months.size(); i++) {
                        String month = months.get(i);
                        Map<String, Object> totalOrderCount = null;
                        for (Map<String, Object> stringObjectMap : totalOrderCountList) {
                            if (stringObjectMap.get("city").equals(city) && stringObjectMap.get("month").equals(month)) {
                                totalOrderCount = stringObjectMap;
                                break;
                            }
                        }
                        if (totalOrderCount == null) {
                            continue;
                        }

                        Long totalOrder = (Long) totalOrderCount.get("all_count");
                        Long rebuyOrder = 0L;
                        for (Map<String, Object> rebuyOrderCount : rebuyOrderCountList) {
                            if (city.equals(rebuyOrderCount.get("city")) && rebuyOrderCount.get("month").equals(month)) {
                                rebuyOrder = (Long) rebuyOrderCount.get("rebuy_count");
                                break;
                            }
                        }
                        if (totalOrder != 0) {
                            cityRebuyRate.set(i, formatDouble((double) rebuyOrder / totalOrder));
                        }
                    }
                    cityRebuyRateMap.put(city, cityRebuyRate);
                    result.add(cityRebuyRateMap);
                }

                return result;
            }
        };
    }

    private String formatDouble(Double d) {
        DecimalFormat df = new DecimalFormat("##0.0000");
        return df.format(d);

    }
}
