package com.jzlife.report.util;

import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;

import java.util.concurrent.Callable;
import java.util.concurrent.Future;

public class ThreadUtil<T> {


    private volatile static ThreadPoolTaskExecutor taskExecutor;


    private ThreadPoolTaskExecutor getTaskExecutor() {
        if (null == taskExecutor) {
            synchronized (ThreadUtil.class) {
                if (null == taskExecutor) {
                    taskExecutor = new ThreadPoolTaskExecutor();
                    taskExecutor.setCorePoolSize(5); //核心线程数
                    taskExecutor.setMaxPoolSize(10); //最大线程数
                    taskExecutor.setQueueCapacity(100); //队列最大长度
                    taskExecutor.setKeepAliveSeconds(60); //线程池维护线程所允许的空闲时间
                    taskExecutor.setThreadNamePrefix("taskExecutor-");
                    taskExecutor.setRejectedExecutionHandler(new java.util.concurrent.ThreadPoolExecutor.CallerRunsPolicy());
                    taskExecutor.initialize();
                }
            }
        }
        return taskExecutor;
    }


    public static void addTask(Runnable task) {
        ThreadUtil threadUtil = new ThreadUtil();
        threadUtil.getTaskExecutor().execute(task);
    }

    public static <T> Future<T> addTask(Callable task) {
        ThreadUtil threadUtil = new ThreadUtil();
        return threadUtil.getTaskExecutor().submit(task);
    }

}
